#   pyRCV2: Preferential vote counting
#   Copyright © 2020  Lee Yingtong Li (RunasSudo)
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

__pragma__ = lambda x: None

from pyRCV2.numbers import Num

class Candidate:
	def __init__(self, name):
		self.name = name
	
	def __repr__(self):
		return '<Candidate ' + self.name + '>'
	
	def toString(self):
		return repr(self)

class CandidateState:
	HOPEFUL = 0
	PROVISIONALLY_ELECTED = 1
	ELECTED = 2
	EXCLUDED = 3
	WITHDRAWN = 4

class Ballot:
	def __init__(self, value, preferences):
		self.value = value
		self.preferences = preferences

class Election:
	"""
	Represents a BLT election
	"""
	
	def __init__(self, name='', seats=0, candidates=None, ballots=None, withdrawn=None):
		self.name = name
		self.seats = seats
		self.candidates = candidates if candidates is not None else []
		self.ballots = ballots if ballots is not None else []
		self.withdrawn = withdrawn if withdrawn is not None else []

class CountCard:
	"""
	Represents a Candidate's (or exhausted pile) current progress in the count
	"""
	def __init__(self):
		self.orig_votes = Num('0')
		self.transfers = Num('0')
		self.ballots = []
		self.state = CandidateState.HOPEFUL
	
	@property
	def votes(self):
		__pragma__('opov')
		return self.orig_votes + self.transfers
		__pragma__('noopov')
	
	def step(self):
		"""Roll over previous round transfers in preparation for next round"""
		self.orig_votes = self.votes
		self.transfers = Num('0')

class CountCompleted:
	pass

class CountStepResult:
	def __init__(self, comment, candidates, exhausted, quota):
		self.comment = comment
		
		self.candidates = candidates # SafeDict: Candidate -> CountCard
		self.exhausted = exhausted # CountCard
		
		self.quota = quota
